var Nodelist = function() {

	this.zones = [];

	this.Flags = {

		// Operating Condition Flags
		'CM' : {
			'description' : "Accepts mail at all times via all methods",
			'mapTo' : "operFlags"
		},
		'ICM' : {
			'description' : "Accepts mail at all times via all TCP/IP methods",
			'mapTo' : "operFlags"
		},
		'MO' : {
			'description' : "Does not accept human callers",
			'mapTo' : "operFlags"
		},
		'LO' : {
			'description' : "Accepts calls only from listed addresses",
			'mapTo' : "operFlags"
		},
		'MN' : {
			'description' : "Will not process compressed mail packets",
			'mapTo' : "operFlags"
		},

		// Modem connection protocols
		'V22' : {
			'description' : "ITU-T V.22",
			'mapTo' : "modems"
		},
		'V29' : {
			'description' : "ITU-T V.29",
			'mapTo' : "modems"
		},
		'V32' : {
			'description' : "ITU-T V.32",
			'mapTo' : "modems"
		},
		'V32b' : {
			'description' : "ITU-T V.32bis",
			'mapTo' : "modems"
		},
		'V34' : {
			'description' : "ITU-T V.34",
			'mapTo' : "modems"
		},
		'V90C' : {
			'description' : "ITU-T V.90 Client",
			'mapTo' : "modems"
		},
		'V90S' : {
			'description' : "ITU-T V.90 Server",
			'mapTo' : "modems"
		},
		'V32T' : {
			'description' : "V.32 Terbo",
			'mapTo' : "modems"
		},
		'VFC' : {
			'description' : "V.Fast Class",
			'mapTo' : "modems"
		},
		'HST' : {
			'description' : "USR Courier HST 9600 bps",
			'mapTo' : "modems"
		},
		'H14' : {
			'description' : "USR Courier HST 14400 bps",
			'mapTo' : "modems"
		},
		'H16' : {
			'description' : "USR Courier HST 16800 bps",
			'mapTo' : "modems"
		},
		'X2C' : {
			'description' : "US Robotics x2 client",
			'mapTo' : "modems"
		},
		'X2S' : {
			'description' : "US Robotics x2 server",
			'mapTo' : "modems"
		},
		'ZYX' : {
			'description' : "Zyxel 16800 bps",
			'mapTo' : "modems"
		},
		'Z19' : {
			'description' : "Zyxel 19200 bps",
			'mapTo' : "modems"
		},
		'H96' : {
			'description' : "Hayes V9600",
			'mapTo' : "modems"
		},
		'PEP' : {
			'description' : "Packet Ensemble Protocol",
			'mapTo' : "modems"
		},
		'CSP' : {
			'description' : "Compucom Speedmodem",
			'mapTo' : "modems"
		},

		// Standards
		'INA' : {
			'description' : "Internet Address",
			'mapTo' : "hostnames",
			'default' : ""
		},
		'IP' : {
			'description' : "IP Capable",
			'mapTo' : "ip",
			'default' : ""
		},
		'IBN' : {
			'description' : "Binkp",
			'mapTo' : "binkp",
			'default' : 24554
		},
		'IFC' : {
			'description' : "RAW ifcico",
			'mapTo' : "raw",
			'default' : 60179
		},
		'IFT' : {
			'description' : "FTP",
			'mapTo' : "ftp",
			'default' : 21
		},
		'ITN' : {
			'description' : "Telnet",
			'mapTo' : "telnet",
			'default' : 23
		},
		"IVM" : {
			'description' : "Vmodem",
			'mapTo' : "vmodem",
			'default' : 3141
		},
		"INO4" : {
			'description' : "No IPv4",
			'mapTo' : "ipv4",
			'default' : false
		},

		// Email
		'IEM' : {
			'description' : "Email Address",
			'mapTo' : "emailAddress",
			'default' : ""
		},
		'ITX' : {
			'description' : "TransX encoding",
			'mapTo' : "transx",
			'default' : ""
		},
		'IUC' : {
			'description' : "uuencoding of mail bundles",
			'mapTo' : "uuencodedMail",
			'default' : ""
		},
		'IMI' : {
			'description' : "MIME encoding of mail bundles",
			'mapTo' : "mimeMail",
			'default' : ""
		},
		'ISE' : {
			'description' : "SEAT protocol",
			'mapTo' : "seat",
			'default' : ""
		},

		// Mail-oriented user flags
		'ZEC' : {
			'description' :  "Zone EchoMail Coordinator",
			'mapTo' : 'userFlags'
		},
		'REC' : {
			'description' :  "Regional EchoMail Coordinator",
			'mapTo' : 'userFlags'
		},
		'NEC' : {
			'description' :  "Network EchoMail Coordinator",
			'mapTo' : 'userFlags'
		},
		'NC' : {
			'description' :  "Network Coordinator",
			'mapTo' : 'userFlags'
		},
		'SDS' : {
			'description' :  "Software Distribution System",
			'mapTo' : 'userFlags'
		},
		'SMH' : {
			'description' :  "Secure Mail Hub",
			'mapTo' : 'userFlags'
		},
		'RPK' :  {
			'description' : "Regional Pointlist Keeper",
			'mapTo' : 'userFlags'
		},
		'NPK' : {
			'description' :  "Network Pointlist Keeper",
			'mapTo' : 'userFlags'
		},
		'ENC' : {
			'description' :  "Accepts encrypted mail",
			'mapTo' : 'userFlags'
		},
		'CDP' : {
			'description' :  "Accepts points auto-created by CD-point",
			'mapTo' : 'userFlags'
		}
	};

	// Probably needs to do more
	var unEscape = function(str) {
		str = str.replace(/_/g, " ");
		return str;
	}

	var searchRecord = function(record, terms) {
		terms = new RegExp(terms, 'i');
		var score = 0;
		for(var property in record) {
			if(Array.isArray(record[property])) {
				for(var e = 0; e < record[property].length; e++) {
					var matches = record[property][e].toString().match(terms);
				}
			} else {
				var matches = record[property].toString().match(terms);
			}
			if(matches !== null)
				score = score + matches.length;
		}
		return score;
	}

	// Returns an array of node records, sorted by frequency of 'terms' occurence
	this.search = function(terms) {
		var results = [];
		// Zones
		for(var z = 0; z < this.zones.length; z++) {
			if(typeof this.zones[z] == "undefined")
				continue;
			// Nodes at zone root
			for(var zn = 0; zn < this.zones[z].nodes.length; zn++) {
				if(typeof this.zones[z].nodes[zn] == "undefined")
					continue;
				var score = searchRecord(this.zones[z].nodes[zn], terms);
				if(score < 1)
					continue;
				results.push(this.zones[z].nodes[zn]);
				results[results.length - 1].score = score;
			}
			// Regions in zone
			for(var zr = 0; zr < this.zones[z].regions.length; zr++) {
				if(typeof this.zones[z].regions[zr] == "undefined")
					continue;
				// Nodes at region root
				for(var rn = 0; rn < this.zones[z].regions[zr].nodes.length; rn++) {
					if(typeof this.zones[z].regions[zr].nodes[rn] == "undefined")
						continue;
					var score = searchRecord(this.zones[z].regions[zr].nodes[rn], terms);
					if(score < 1)
						continue;
					results.push(this.zones[z].regions[zr].nodes[rn]);
					results[results.length - 1].score = score;
				}
				// Hosts in region
				for(var rh = 0; rh < this.zones[z].regions[zr].hosts.length; rh++) {
					if(typeof this.zones[z].regions[zr].hosts[rh] == "undefined")
						continue;
					// Nodes per host
					for(var hn = 0; hn < this.zones[z].regions[zr].hosts[rh].nodes.length; hn++) {
						if(typeof this.zones[z].regions[zr].hosts[rh].nodes[hn] == "undefined")
							continue;
						var score = searchRecord(this.zones[z].regions[zr].hosts[rh].nodes[hn], terms);
						if(score < 1)
							continue;
						results.push(this.zones[z].regions[zr].hosts[rh].nodes[hn]);
						results[results.length - 1].score = score;
					}
				}
			}
		}
		results.sort(
			function(a, b) {
				return b.score - a.score;
			}
		);
		return results;
	}

	// Assumes an argument to something like "/path/to/my/nodelist.289"
	this.load = function(file, append) {

		if(!file_exists(file))
			throw "File not found: " + file;

		var f = new File(file);
		f.open("r");

		if(!f.is_open)
			throw "Unable to open " + file;

		if(typeof append == "undefined" || !append)
			this.zones = [];

		var state = {
			'zone' : 0,
			'region' : 0,
			'host' : 0
		};

		while(!f.eof) {

			var line = f.readln();

			if(line === null)
				continue;

			if(line.match(/^;/) !== null)
				continue;

			line = line.split(",");
			if(line.length < 8)
				continue;

			var record = {
				'keyword' : line[0].toUpperCase(),
				'zone' : 0,
				'host' : 0,
				'number' : parseInt(line[1]),
				'name' : unEscape(line[2]),
				'location' : unEscape(line[3]),
				'sysop' : unEscape(line[4]),
				'phoneNumber' : line[5],
				'baudRate' : line[6],
				'hostnames' : [],
				'modems' : [],
				'operFlags' : [],
				'userFlags' : []
			};

			for(var l = 7; l < line.length; l++) {

				if(line[l].toUpperCase() == "U" || line[l] == "")
					continue;

				var flag = line[l].split(":");
				if(typeof this.Flags[flag[0]] == "undefined") {
					record.userFlags.push(line[l]);
					continue;
				}

				if(flag.length > 1)
					var value = flag[1];
				else if(flag.length == 1 && typeof this.Flags[flag[0]].default != "undefined")
					var value = this.Flags[flag[0]].default;
				else
					var value = flag[0];

				if(Array.isArray(record[this.Flags[flag[0]].mapTo]))
					record[this.Flags[flag[0]].mapTo].push(value);
				else
					record[this.Flags[flag[0]].mapTo] = value;

				if(record.hostnames.length < 1)
					record.ipv4 = false;

			}

			switch(record.keyword) {
				case "ZONE":
					state.zone = record.number;
					state.region = 0;
					state.host = 0;
					this.zones[state.zone] = {
						'details' : record,
						'nodes' : [],
						'regions' : []
					};
					break;
				case "REGION":
					state.region = record.number;
					state.host = 0;
					this.zones[state.zone].regions[state.region] = {
						'details' : record,
						'nodes' : [],
						'hosts' : []
					};
					break;
				case "HOST":
					state.host = record.number;
					this.zones[state.zone].regions[state.region].hosts[state.host] = {
						'details' : record,
						'nodes' : []
					};
					break;
				default:
					record.zone = state.zone;
					if(state.region == 0) {
						record.host = 1;
						this.zones[state.zone].nodes[record.number] = record;
					} else if(state.host == 0) {
						record.host = state.region;
						this.zones[state.zone].regions[state.region].nodes[record.number] = record;
					} else {
						record.host = state.host;
						this.zones[state.zone].regions[state.region].hosts[state.host].nodes[record.number] = record;
					}
					break;
			}

		}

		f.close();

	}

	// Assumes an argument like "/path/to/my/nodelist.*"
	this.loadNewest = function(path, append) {
		var files = directory(path);
		var newest = { 'path' : "", 'time' : 0 };
		files.forEach(
			function(nodelist) {
				var ut = file_utime(nodelist);
				if(ut > newest.time)
					newest = { 'path' : nodelist, 'time' : ut };
			}
		);
		if(newest.path != "")
			this.load(newest.path, append);
	}

}