load('sbbsdefs.js');
load('frame.js');
load('tree.js');
load('inputline.js');
load('ftn-nodelist.js');

// The paths to your nodelists
var nodelistPaths = [
	"/sbbs/xtrn/ftnnv/nodelist.*"
];

// This entire thing is horrible
var Browser = function(parentFrame) {

	var STATE_LIST = 0,
		STATE_POPUP = 1,
		STATE_SEARCH_INPUT = 2,
		STATE_SEARCH_RESULTS = 3,
		STATE_SEARCH_POPUP = 4;

	var frame,
		subFrame,
		popUpFrame,
		searchInputFrame,
		searchFrame,
		tree,
		searchResultsTree,
		nodelist,
		inputLine,
		state = STATE_LIST;

	var colors = {
		// non-current item foreground
		fg:LIGHTGRAY,
		// non-current item/empty space background 
		bg:BG_BLACK,
		// current item foreground
		lfg:WHITE,
		// current item background
		lbg:BG_CYAN,
		// current tree heading foreground
		cfg:LIGHTGRAY,
		// current tree heading background
		cbg:BG_BLACK,
		// tree branch foreground
		tfg:LIGHTCYAN,
		// tree heading foreground
		hfg:LIGHTGRAY,
		// tree heading background
		hbg:BG_BLACK,
		// tree expansion foreground
		xfg:LIGHTCYAN
	}

	var formatItem = function(record) {
		var item = format(
			"%d:%d/%d %s",
			record.zone, record.host, record.number, record.name
		);
		return item;
	}

	var nodeDetails = function(record) {

		popUpFrame = new Frame(
			frame.x + 1,
			frame.y + Math.floor((frame.height - 12) / 2),
			frame.width - 2,
			12,
			BG_BLUE|WHITE,
			frame
		);

		popUpSubFrame = new Frame(
			popUpFrame.x + 1,
			popUpFrame.y + 1,
			popUpFrame.width - 2,
			popUpFrame.height - 2,
			WHITE,
			popUpFrame
		);

		var putLine = function(field, value) {
			if(Array.isArray(value))
				value = value.join(", ");
			value = value.substr(0, popUpSubFrame.width - 17);
			popUpSubFrame.putmsg(format("%15s: %s\r\n", field, value));
		}

		popUpFrame.center("Node Details");
		popUpFrame.gotoxy(1, popUpFrame.height);
		popUpFrame.center("Press any key to close");

		putLine("Address", record.zone + ":" + record.host + "/" + record.number);
		putLine("Name", record.name);
		putLine("Location", record.location);
		putLine("Sysop", record.sysop);

		if(record.hostnames.length > 0)
			putLine("Hostnames", record.hostnames);

		if(record.phoneNumber != "" && record.phoneNumber.toUpperCase() != "-UNPUBLISHED-")
			putLine("Phone", record.phoneNumber);

		if(record.modems.length > 0)
			putLine("Modems", record.modems);

		if(record.operFlags.length > 0)
			putLine("Operating Flags", record.operFlags);

		if(record.userFlags.length > 0)
			putLine("User Flags", record.userFlags);

		if(typeof record.emailAddress != "undefined")
			putLine("Email", record.emailAddress);

		popUpFrame.open();
		state = (state == STATE_LIST) ? STATE_POPUP : STATE_SEARCH_POPUP;

	}

	var openSearchInput = function() {

		searchInputFrame = new Frame(
			frame.x + 1,
			frame.y + Math.floor((frame.height - 3) / 2),
			frame.width - 2,
			3,
			BG_BLUE|WHITE,
			frame
		);

		var searchInputSubFrame = new Frame(
			searchInputFrame.x + 1,
			searchInputFrame.y + 1,
			searchInputFrame.width - 2,
			1,
			BG_BLACK|WHITE,
			searchInputFrame
		);

		searchInputFrame.center("Search by BBS name, sysop, location, hostname, phone number, etc.");

		inputLine = new InputLine(searchInputSubFrame);
		inputLine.show_cursor = true;

		searchInputFrame.open();
		inputLine.open();

	}

	var closeSearchInput = function() {
		searchInputFrame.close();
		inputLine = undefined;
	}

	var displaySearchResults = function(results) {

		searchFrame = new Frame(
			subFrame.x,
			subFrame.y,
			subFrame.width,
			subFrame.height,
			WHITE,
			subFrame
		);

		searchResultsTree = new Tree(searchFrame);
		for(var c in colors) {
			searchResultsTree.colors[c] = colors[c];
		}

		if(results.length < 1) {
			searchResultsTree.addItem("No results found.");
		} else {
			results.forEach(
				function(result) {
					searchResultsTree.addItem(
						formatItem(result),
						function() {
							nodeDetails(result)
						}					
					);
				}
			);
		}

		searchFrame.open();
		searchResultsTree.open();

		frame.gotoxy(1, frame.height);
		frame.putmsg("Displaying search results. Hit Q to quit.");

	}

	var initDisplay = function() {

		frame = new Frame(
			1,
			1,
			console.screen_columns,
			console.screen_rows,
			BG_BLUE|WHITE
		);

		subFrame = new Frame(
			frame.x,
			frame.y + 1,
			frame.width,
			frame.height - 2,
			WHITE,
			frame
		);

		var treeFrame = new Frame(
			subFrame.x,
			subFrame.y,
			subFrame.width,
			subFrame.height,
			subFrame.attr,
			subFrame
		);

		tree = new Tree(treeFrame);
		for(var c in colors) {
			tree.colors[c] = colors[c];
		}

		frame.open();
		tree.open();

		frame.center("FTN Nodelist Viewer");
		frame.gotoxy(1, frame.height);
		frame.putmsg("S)earch, Q)uit");

	}

	var initData = function(file) {
		nodelist = new Nodelist();
		nodelistPaths.forEach(
			function(nlp) {
				nodelist.loadNewest(nlp, true);
			}
		);
	}

	// So very sloppy :D
	var initMenu = function() {
		// List zones
		for(var z = 0; z < nodelist.zones.length; z++) {
			if(typeof nodelist.zones[z] == "undefined")
				continue;
			var zoneTree = tree.addTree(
				"Zone " +
				z +
				": " +
				nodelist.zones[z].details.name
			);
			// List nodes at zone root
			for(var zn = 0; zn < nodelist.zones[z].nodes.length; zn++) {
				if(typeof nodelist.zones[z].nodes[zn] == "undefined")
					continue;
				zoneTree.addItem(
					formatItem(nodelist.zones[z].nodes[zn]),
					(function(node, zone, net) {
						return function() {
							nodeDetails(node);
						}
					})(nodelist.zones[z].nodes[zn], z, 1)
				);
			}
			// List regions in zone
			for(var zr = 0; zr < nodelist.zones[z].regions.length; zr++) {
				if(typeof nodelist.zones[z].regions[zr] == "undefined")
					continue;
				// List nodes at region root
				var regionTree = zoneTree.addTree(
					"Region " +
					nodelist.zones[z].regions[zr].details.number +
					": " +
					nodelist.zones[z].regions[zr].details.name
				);
				for(var rn = 0; rn < nodelist.zones[z].regions[zr].nodes.length; rn++) {
					if(typeof nodelist.zones[z].regions[zr].nodes[rn] == "undefined")
						continue;
					regionTree.addItem(
						formatItem(nodelist.zones[z].regions[zr].nodes[rn]),
						(function(node) {
							return function() { nodeDetails(node); }
						})(nodelist.zones[z].regions[zr].nodes[rn])
					);
				}
				// List hosts (nets) in region
				for(var rh = 0; rh < nodelist.zones[z].regions[zr].hosts.length; rh++) {
					if(typeof nodelist.zones[z].regions[zr].hosts[rh] == "undefined")
						continue;
					var hostTree = regionTree.addTree(
						"Network " +
						nodelist.zones[z].regions[zr].hosts[rh].details.number +
						": " +
						nodelist.zones[z].regions[zr].hosts[rh].details.name
					);
					// List nodes in host (net)
					for(var hn = 0; hn < nodelist.zones[z].regions[zr].hosts[rh].nodes.length; hn++) {
						if(typeof nodelist.zones[z].regions[zr].hosts[rh].nodes[hn] == "undefined")
							continue;
						hostTree.addItem(
							formatItem(nodelist.zones[z].regions[zr].hosts[rh].nodes[hn]),
							(function(node) {
								return function() { nodeDetails(node); }
							})(nodelist.zones[z].regions[zr].hosts[rh].nodes[hn])
						);
					}
				}
			}
		}
	}

	var init = function() {
		initData();
		initDisplay();
		initMenu();
	}

	this.cycle = function() {
		if(frame.cycle())
			console.gotoxy(console.screen_columns, console.screen_rows);
	}

	this.close = function() {
		tree.close();
		frame.close();
	}

	init();

	this.getcmd = function() {
		var ret = true;
		if(state == STATE_SEARCH_INPUT) {
			var res = inputLine.getkey();
			if(typeof res != "undefined") {
				state = STATE_SEARCH_RESULTS;
				closeSearchInput();
				var results = nodelist.search(res);
				displaySearchResults(results);
			}
		} else {

			var cmd = console.inkey(K_NONE, 5).toUpperCase();

			switch(state) {
				case STATE_LIST:
					if(cmd == "Q") {
						ret = false;
					} else if(cmd == "S") {
						state = STATE_SEARCH_INPUT;
						openSearchInput();
					} else {
						tree.getcmd(cmd);
					}
					break;
				case STATE_POPUP:
					if(cmd != "") {
						popUpFrame.close();
						popUpFrame.delete();
						state = STATE_LIST;
					}
					break;
				case STATE_SEARCH_RESULTS:
					if(cmd == "Q") {
						state = STATE_LIST;
						searchFrame.close();
						searchResultsTree.close();
						searchFrame.delete();
						searchResultsTree = null;
						frame.gotoxy(1, frame.height);
						frame.clearline();
						frame.putmsg("S)earch, Q)uit");
					} else {
						searchResultsTree.getcmd(cmd);
					}
					break;
				case STATE_SEARCH_POPUP:
					if(cmd != "") {
						popUpFrame.close();
						popUpFrame.delete();
						state = STATE_SEARCH_RESULTS;
					}
					break;
				default:
					break;
			}

		}
		return ret;

	}

}

var sysStatus, attr;

var init = function() {
	sysStatus = bbs.sys_status;
	bbs.sys_status|=SS_MOFF;
	attr = console.attributes;
	console.clear(LIGHTGRAY);
}

var main = function() {
	var browser = new Browser();
	while(!js.terminated) {
		if(!browser.getcmd())
			break;
		browser.cycle();
	}
	browser.close();
}

var cleanUp = function() {
	bbs.sys_status = sysStatus;
	console.attributes = attr;
	console.clear(attr);
}

init();
main();
cleanUp();
